local utils = require 'utils'
local cjson = require 'cjson.safe'
local errType = require 'errType'
local config = require 'config'

local ArcView_DbPSqlConnectionCheck = {}

function ArcView_DbPSqlConnectionCheck.invoke(args)
    local host, port, user,
    password, name, schema, sudo

    sudo = false

    if not (config and config.arcview and config.arcview.location) then
        return nil, errType.codes.InvalidParams,
        {message = 'Invalid arcview configuration.'}
    end

    if args ~= nil then
        if utils.isArray(args) then
            host = args[1]
            port = args[2]
            user = args[3]
            password = args[4]
            name = args[5]
            schema = args[6]
        else
            sudo = args.sudo
            host = args.host
            port = args.port
            user = args.user
            password = args.password
            name = args.name
            schema = args.schema
        end
    end

    if sudo ~= nil and type(sudo) ~= 'boolean' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'sudo\' must be a boolean.'}
    end
    if host ~= nil and type(host) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'host\' must be a string.'}
    end
    if user ~= nil and type(user) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'user\' must be a string.'}
    end
    if port ~= nil and utils.isInt(port) == false then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'port\' must be an int.'}
    end
    if password ~= nil and type(password) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'password\' must be a string.'}
    end
    if name ~= nil and type(name) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'name\' must be a string.'}
    end
    if schema ~= nil and type(schema) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'schema\' must be a string.'}
    end

    local rexarcParams = {}

    if host ~= nil then
        table.insert(rexarcParams, '--host')
        table.insert(rexarcParams, host)
    end
    if port ~= nil then
        table.insert(rexarcParams, '--port')
        table.insert(rexarcParams, tostring(port))
    end
    if user ~= nil then
        table.insert(rexarcParams, '--user')
        table.insert(rexarcParams, user)
    end
    if password ~= nil then
        table.insert(rexarcParams, '--password')
        table.insert(rexarcParams, password)
    end
    if name ~= nil then
        table.insert(rexarcParams, '--name')
        table.insert(rexarcParams, name)
    end
    if schema ~= nil then
        table.insert(rexarcParams, '--schema')
        table.insert(rexarcParams, schema)
    end

    local app_dir = config.arcview.location
    local out, err, errData
    if sudo then
        out, err, errData = utils.shell_posix('sudo', nil, '-u', 'postgres', '/usr/bin/python3', app_dir .. 'manage_bootstrap.py', 'database_postgresql_connection_check', unpack(rexarcParams))
    else
        out, err, errData = utils.shell_posix('python3', nil, app_dir .. 'manage_bootstrap.py', 'database_postgresql_connection_check', unpack(rexarcParams))
    end
    if err ~= nil then
        return nil, err, errData
    end

    return out
end

return ArcView_DbPSqlConnectionCheck
