local utils = require 'utils'
local cjson = require 'cjson.safe'
local errType = require 'errType'
local config = require 'config'

local ArcView_DbPSqlSchemaInit = {}

function ArcView_DbPSqlSchemaInit.invoke(args)
    local database

    if not (config and config.arcview and config.arcview.location) then
        return nil, errType.codes.InvalidParams,
        {message = 'Invalid arcview configuration.'}
    end

    if args ~= nil then
        if utils.isArray(args) then
            database = args[1]
        else
            database = args.database
        end
    end

    if database == nil or type(database) ~= 'string' then
        return nil, errType.codes.InvalidParams,
        {message = 'Parameter \'database\' is mandatory and must be a string.'}
    end

    local rexarcParams = {}

    if database ~= nil then
        table.insert(rexarcParams, '--database')
        table.insert(rexarcParams, database)
    end

    local app_dir = config.arcview.location
    local out, err, errData = utils.shell_posix('python3', nil, app_dir .. 'manage_bootstrap.py', 'database_postgresql_schema_init', unpack(rexarcParams))
    if err ~= nil then
        return nil, err, errData
    end

    return out
end

return ArcView_DbPSqlSchemaInit
