local errType = require 'errType'
local utils = require 'utils'
local move = require 'methods/FileSystem_Move'

local Certificate_Write = {}

function Certificate_Write.invoke(args)
    local key
    local cert
    local rexKeyPath = '/rex/rexcore/rexcore.key'
    local rexCrtPath = '/rex/rexcore/rexcore.cer'

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                key = args[1]
            end
            if #args >= 2 then
                cert = args[2]
            end
        else
            key = args.key
            cert = args.cert
        end
    end

    if key ~= nil and type(key) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Invalid parameter \'key\'.'}
    end
    if cert ~= nil and type(cert) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Invalid parameter \'cert\'.'}
    end
    if cert == nil and key == nil then
        return nil, errType.codes.InvalidParams, {message = 'No parameters given.'}
    end

    -- store keys in temp
    local keyPath
    if key then
        keyPath = '/tmp/rexcore.key'
        local res = utils.writeToFile(keyPath, key)
        if res ~= true then
            return nil, errType.codes.ServerError, {message = 'Failed to save temporary file \'/tmp/rexcore.key\'.'}
        end
    else
        keyPath = rexKeyPath
    end

    local crtPath
    if key then
        crtPath = '/tmp/rexcore.cer'
        local res = utils.writeToFile(crtPath, cert)
        if res ~= true then
            return nil, errType.codes.ServerError, {message = 'Failed to save temporary file \'/tmp/rexcore.cer\'.'}
        end
    else
        crtPath = rexCrtPath
    end

    -- check compatibility
    local stdout, err, errData = utils.shell_posix('openssl', nil, 'rsa', '-in', keyPath, '-modulus', '-noout')
    if err ~= nil then
        errData.message = errData.message .. '\n Private key is not valid.'
        return nil, err, errData
    end
    local keyModulus = utils.trim(stdout)

    stdout, err, errData = utils.shell_posix('openssl', nil, 'x509', '-in', crtPath, '-modulus', '-noout')
    if err ~= nil then
        errData.message = errData.message .. '\n Certificate is not valid.'
        return nil, err, errData
    end
    local crtModulus = utils.trim(stdout)

    if keyModulus ~= crtModulus then
        return nil, errType.codes.InvalidParams, {message = 'Private key and certificate do not match.'}
    end

    -- update keys
    if key then
        local arg = {}
        arg.oldPath = keyPath
        arg.newPath = rexKeyPath
        local _
        _, err, errData = move.invoke(arg)
        if err ~= nil then
            return nil, err, errData
        end
    end
    if cert then
        local arg = {}
        arg.oldPath = crtPath
        arg.newPath = rexCrtPath
        local _
        _, err, errData = move.invoke(arg)
        if err ~= nil then
            return nil, err, errData
        end
    end
end

return Certificate_Write
