local errType = require 'errType'
local utils = require 'utils'

local FileSystem_List = {}

function FileSystem_List.invoke(args)
    local path

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                path = args[1]
            end
        else
            path = args.path
        end
    end

    if path == nil or type(path) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'path\'.'}
    end

    local readable, err, errData = utils.isPathReadable(path)
    if readable ~= true then
        return nil, err, errData
    end

    local stdout
    stdout, err, errData = utils.shell_posix('ls', nil, '-l1a', '--time-style=+%Y-%m-%d %H:%M:%S', path)
    if err ~= nil then
        return nil, err, errData
    end

    local result = {}

    local lines = utils.split(stdout, '\n')
    for _, line in pairs(lines) do
        local permissions, numberOfLinks, owner, group, size, modifiedDate, modifiedTime, name
        = string.match(line, '([%w-]+)%s+(%d+)%s+(%S+)%s+(%S+)%s+(%d+)%s+([%w|-]+)%s+([%w|:]+)%s+(.+)%s*')

        if name ~= nil then
            local item = {}
            if string.sub(permissions, 1, 1) == 'd' then
                item.directory = true
            end
            if string.sub(permissions, 1, 1) == 'l' then
                item.link = true
            end
            item.permissions = string.sub(permissions, 2)
            item.numberOfLinks = numberOfLinks
            item.owner = owner
            item.group = group
            item.size = tonumber(size)
            item.modified = modifiedDate .. ' ' .. modifiedTime
            item.name = name

            table.insert(result, item)
        end
    end

    return result
end

return FileSystem_List
