local errType = require 'errType'
local utils = require 'utils'

local FileSystem_Move = {}

function FileSystem_Move.invoke(args)
    local oldPath
    local newPath

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                oldPath = args[1]
            end
            if #args >= 2 then
                newPath = args[2]
            end
        else
            oldPath = args.oldPath
            newPath = args.newPath
        end
    end

    if oldPath == nil or type(oldPath) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'oldPath\'.'}
    end
    if newPath == nil or type(newPath) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'newPath\'.'}
    end

    local readable, err, errData = utils.isPathReadable(oldPath)
    if readable ~= true then
        return nil, err, errData
    end

    local writable
    writable, err, errData = utils.isPathWritable(oldPath)
    if writable ~= true then
        return nil, err, errData
    end

    writable, err, errData = utils.isPathWritable(newPath)
    if writable ~= true then
        return nil, err, errData
    end

    local _
    _, err, errData = utils.shell_posix('mv', nil, oldPath, newPath)
    if err ~= nil then
        return nil, err, errData
    end
end

return FileSystem_Move
