local errType = require 'errType'
local utils = require 'utils'

local FileSystem_SetFileContent = {}

function FileSystem_SetFileContent.invoke(args)
    local path
    local content

    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                path = args[1]
            end
            if #args >= 2 then
                content = args[2]
            end
        else
            path = args.path
            content = args.content
        end
    end

    if path == nil or type(path) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'path\'.'}
    end
    if content == nil or type(content) ~= 'string' then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'content\'.'}
    end

    local writable, err, errData = utils.isPathWritable(path)
    if writable ~= true then
        return nil, err, errData
    end

    local res = utils.writeToFile(path, content)
    if res ~= true then
        return nil, errType.codes.ServerError, {message = 'Failed to write to path \'' .. path .. '\'.'}
    end
end

return FileSystem_SetFileContent
