local errType = require 'errType'
local utils = require 'utils'

local Hostname_SetConfig = {}

function Hostname_SetConfig.isHostnameValid(hostname)
    if hostname == nil then
        return false
    end
    if type(hostname) ~= 'string' then
        return false
    end
    return true
end

function Hostname_SetConfig.invoke(args)
    local hostname = nil

    if args ~= nil then
        if utils.isArray(args) and #args == 1 then
            hostname = args[1]
        else
            hostname = args.hostname
        end
    end

    local valid = Hostname_SetConfig.isHostnameValid(hostname)

    if valid == false then
        return nil, errType.codes.InvalidParams, {message = 'Failed to read parameter \'hostname\'.'}
    end

    local content = utils.readFromFile('/etc/hosts')
    if content == nil then
        return nil, errType.codes.ServerError, {message = 'Failed to read file \'/etc/hosts\'.'}
    end

    content = utils.trim(content)

    local count
    content, count = string.gsub(content, "127%.0%.1%.1[^\n$]*", '127.0.1.1 ' .. hostname .. '\n')
    if count == 0 then
        content = content .. '\n127.0.1.1 ' .. hostname .. '\n'
    end

    local _, err, errData = utils.shell_posix('sudo', content, 'dd', 'of=/etc/hosts')
    if err ~= nil and errData.exitcode ~= 0  then
        return nil, errType.codes.ServerError, {message = 'Failed to write to file \'/etc/hosts\'.'}
    end

    _, err, errData = utils.shell_posix('sudo', hostname .. '\n', 'dd', 'of=/etc/hostname')
    if err ~= nil and errData.exitcode ~= 0 then
        return nil, errType.codes.ServerError, {message = 'Failed to write to file \'/etc/hostname\'.'}
    end

    _, err, errData = utils.shell_posix('sudo', nil, 'hostname', hostname)
    if err ~= nil and errData.exitcode ~= 0 then
        return nil, err, errData
    end

    _, err, errData = utils.shell_posix('sudo', nil, 'hostnamectl', 'set-hostname', hostname)
    if err ~= nil and errData.exitcode ~= 0 then
        return nil, err, errData
    end
    return nil
end

return Hostname_SetConfig
