local errType = require 'errType'
local utils = require 'utils'

local TimeDate_GetState = {}

function TimeDate_GetState.invoke()

    local content = utils.readFromFile('/proc/uptime')
    if content == nil then
        return nil, errType.codes.ServerError, {message = 'Failed to read content from path \'/proc/uptime\'.'}
    end

    local uptime = {}
    local up, idle = string.match(content, '(%d+%.%d+)%s+(%d+%.%d+)')
    if up ~= nil then
        uptime.up = tonumber(up)
    end
    if idle ~= nil then
        uptime.idle = tonumber(idle)
    end

    local stdout, err, errData = utils.shell_posix('timedatectl', nil, 'status')
    if err ~= nil then
        return nil, err, errData
    end

    local lines = utils.split(stdout, '\n')
    for _, line in pairs(lines) do
        local param, value = string.match(line, '%s*([%w|%s]+):(.+)%s*')
        if param ~= nil and value ~= nil then
            param = utils.trim(param)
            value = utils.trim(value)

            if (param == 'Local time') then
                uptime.localTime = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end

            if (param == 'Universal time') then
                uptime.universalTime = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end

            if (param == 'RTC time') then
                uptime.rtc = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end

            if (param == 'Time zone') then
                uptime.zone = utils.trim(string.match(value, '([%w|/]+)%s'))
            end

            if (param == 'NTP service') then
                uptime.ntpEnabled = utils.trim(value)
                if uptime.ntpEnabled == 'active' then
                    uptime.ntpEnabled = true
                else
                    uptime.ntpEnabled = false
                end
            end

            if (param == 'System clock synchronized') then
                uptime.ntpSynchronized = utils.trim(value)
                if uptime.ntpSynchronized == 'yes' then
                    uptime.ntpSynchronized = true
                else
                    uptime.ntpSynchronized = false
                end
            end

            if (param == 'RTC in local TZ') then
                uptime.rtcInLocal = utils.trim(value)
                if uptime.rtcInLocal == 'yes' then
                    uptime.rtcInLocal = true
                else
                    uptime.rtcInLocal = false
                end
            end
        end
    end

    local _
    stdout, err, _ = utils.shell_posix('systemctl', nil, 'status', 'systemd-timesyncd')
    if err == nil then
        lines = utils.split(stdout, '\n')
        for _, line in pairs(lines) do
            local param, value = string.match(line, '%s*([%w|%s]+):(.+)%s*')
            if param ~= nil and value ~= nil then
                param = utils.trim(param)
                value = utils.trim(value)

                if (param == 'Status') then
                    uptime.ntpStatus =  value:match "^\"(.-)\"$"
                    break;
                end
            end
        end
    end

    local content = utils.readFromFile('/etc/systemd/timesyncd.conf')
    if content == nil then
        return nil, errType.codes.ServerError, {message = 'Failed to read content from path \'/etc/systemd/timesyncd.conf\'.'}
    end

    lines = utils.split(content, '\n')
    for _, line in pairs(lines) do
        local ntpServers = string.match(line, 'Servers%s*=%s*(.*)')
        if ntpServers ~= nil and ntpServers ~= '' then
            uptime.ntpServers = utils.split(ntpServers, ' ')
        end
        ntpServers = string.match(line, '^NTP%s*=%s*(.*)')
        if ntpServers ~= nil and ntpServers ~= '' then
            uptime.ntpServers = utils.split(ntpServers, ' ')
        end
    end

    return uptime
end

return TimeDate_GetState
